import qbs.File
import qbs.TextFile

Project
{   // Side note about another unrelated issue - https://bugreports.qt.io/browse/QBS-1661
    // In order to set the 'seven_zip_path' property from Build.qbs
    // I had to wrap this Product in Project item.
    // It would be much more convenient to set this property directly from the Build.qbs
    // without this redundant wrapping in Project.

    property string seven_zip_path

    InstallPackage
    {
        property string seven_zip_path: project.seven_zip_path
        property string installRoot: qbs.installRoot // A hack, see Artifact's comment for explanation

        Depends { name: "Program" }

        builtByDefault: true
        type: base.concat("dummy") // A hack, see Artifact's comment for explanation
        archiver.type: "7zip"
        archiver.command: seven_zip_path

        Rule
        {   // At this point the archive with redundant "Program/..." paths has already been created.
            // This rule fixes it by renaming all paths in the archive from "Program/file" to just "file"
            // The inputs are archive created by InstallPackage (archiver.archive) and
            // .tarlist text file (archiver.input-list) containing relative paths.
            // .tarlist is read line by line and "Program/" is removed.
            // This is then used to rename files in the archive.

            Artifact
            {   // This is a nonexistent file,
                // this artifact was added simply because QBS makes it imperative to do so.
                // I could not make the edited 7z archive to be the artifact
                // (got the "Cycle detected in rule dependencies" error),
                // so I had to copy it to install-root from the prepare script.
                // And in order to do so I had to define property string installRoot: qbs.installRoot
                // Anyway, this is irrelevant to the issue, just explaining why such hack is used.
                filePath: product.buildDirectory + "/Dummy.txt"
                fileTags: "dummy"
            }

            multiplex: true
            inputs: ["archiver.archive", "archiver.input-list"]

            prepare:
            {
                var commands = []

                var archive = inputs["archiver.archive"]
                var archive_path = archive[0].filePath

                var input_list = inputs["archiver.input-list"]
                var tarlist = new TextFile (input_list[0].filePath)

                // ----------------
                // Rename files within the archive
                // ----------------

                while (!tarlist.atEof())
                {
                    var original = tarlist.readLine()
                    var modified = original.replace("Program/","")
                    var command  = new Command (product.seven_zip_path, ["rn", archive_path, original, modified])
                    command.description = "Renaming " + original
                    commands.push(command)
                }
                tarlist.close()

                // ----------------
                // Copy the archive to install-root/Archive-for-installer
                // ----------------

                var copy_archive = new JavaScriptCommand()
                copy_archive.description = "Copying the archive to " + product.installRoot + "/Archive-for-installer"
                copy_archive.sourceCode = function()
                {
                    var archive = inputs["archiver.archive"]
                    var old_path  = archive[0].filePath
                    var new_path  = product.installRoot + "/Archive-for-installer"
                    File.makePath(new_path)
                    File.copy(old_path, new_path + "/Archive-for-installer.7z")
                }
                commands.push(copy_archive)

                return commands

                // All of this hackery could be avoided
                // if it would be possible to edit the list of file paths to be archived in advance.
                // Perhaps add property 'removeFromPaths' to the Archiver?
                // User could set archiver.removeFromPaths: "Program/"
                // and then it would perform path.replace("Program/","") for every file path.
            }
        }
    }
}
